@@ -1,6 +1,5 @@
-# -*- encoding: utf-8 -*-
 #
-# Copyright © 2013 Red Hat, Inc
+# Copyright 2013 Red Hat, Inc
 #
 # Author:  Eoghan Glynn <eglynn@redhat.com>
 #
@@ -20,6 +19,7 @@ import warnings
 
 from ceilometerclient.common import base
 from ceilometerclient.common import utils
+from ceilometerclient import exc
 from ceilometerclient.v2 import options
 
 
@@ -36,7 +36,8 @@ UPDATABLE_ATTRIBUTES = [
     'threshold_rule',
     'combination_rule',
 ]
-CREATION_ATTRIBUTES = UPDATABLE_ATTRIBUTES + ['project_id', 'user_id']
+CREATION_ATTRIBUTES = UPDATABLE_ATTRIBUTES + ['project_id', 'user_id',
+                                              'time_constraints']
 
 
 class Alarm(base.Resource):
@@ -48,8 +49,17 @@ class Alarm(base.Resource):
         # that look like the Alarm storage object
         if k == 'rule':
             k = '%s_rule' % self.type
+        if k == 'id':
+            return self.alarm_id
         return super(Alarm, self).__getattr__(k)
 
+    def delete(self):
+        return self.manager.delete(self.alarm_id)
+
+    def get_state(self):
+        state = self.manager.get_state(self.alarm_id)
+        return state.get('alarm')
+
 
 class AlarmChange(base.Resource):
     def __repr__(self):
@@ -75,6 +85,14 @@ class AlarmManager(base.Manager):
         except IndexError:
             return None
 
+        except exc.HTTPNotFound:
+            # When we try to get deleted alarm HTTPNotFound occurs
+            # or when alarm doesn't exists this exception don't must
+            # go deeper because cleanUp() (method which remove all
+            # created things like instance, alarm, etc.) at scenario
+            # tests doesn't know how to process it
+            return None
+
     @classmethod
     def _compat_legacy_alarm_kwargs(cls, kwargs, create=False):
         cls._compat_counter_rename_kwargs(kwargs, create)
@@ -111,6 +129,26 @@ class AlarmManager(base.Manager):
             del kwargs['matching_metadata']
             kwargs['threshold_rule']['query'] = query
 
+    @staticmethod
+    def _merge_time_constraints(existing_tcs, kwargs):
+        new_tcs = kwargs.get('time_constraints', [])
+        if not existing_tcs:
+            updated_tcs = new_tcs
+        else:
+            updated_tcs = [dict(tc) for tc in existing_tcs]
+            for tc in new_tcs:
+                for i, old_tc in enumerate(updated_tcs):
+                    if old_tc['name'] == tc['name']:  # if names match, merge
+                        utils.merge_nested_dict(updated_tcs[i], tc)
+                        break
+                else:
+                    updated_tcs.append(tc)
+        tcs_to_remove = kwargs.get('remove_time_constraints', [])
+        for tc in updated_tcs:
+            if tc['name'] in tcs_to_remove:
+                updated_tcs.remove(tc)
+        return updated_tcs
+
     def create(self, **kwargs):
         self._compat_legacy_alarm_kwargs(kwargs, create=True)
         new = dict((key, value) for (key, value) in kwargs.items()
@@ -119,7 +157,12 @@ class AlarmManager(base.Manager):
 
     def update(self, alarm_id, **kwargs):
         self._compat_legacy_alarm_kwargs(kwargs)
-        updated = self.get(alarm_id).to_dict()
+        alarm = self.get(alarm_id)
+        if alarm is None:
+            raise exc.CommandError('Alarm not found: %s' % alarm_id)
+        updated = alarm.to_dict()
+        updated['time_constraints'] = self._merge_time_constraints(
+            updated.get('time_constraints', []), kwargs)
         kwargs = dict((k, v) for k, v in kwargs.items()
                       if k in updated and k in UPDATABLE_ATTRIBUTES)
         utils.merge_nested_dict(updated, kwargs, depth=1)
@@ -129,14 +172,12 @@ class AlarmManager(base.Manager):
         return self._delete(self._path(alarm_id))
 
     def set_state(self, alarm_id, state):
-        resp, body = self.api.json_request('PUT',
-                                           "%s/state" % self._path(alarm_id),
-                                           body=state)
+        body = self.api.put("%s/state" % self._path(alarm_id),
+                            json=state).json()
         return body
 
     def get_state(self, alarm_id):
-        resp, body = self.api.json_request('GET',
-                                           "%s/state" % self._path(alarm_id))
+        body = self.api.get("%s/state" % self._path(alarm_id)).json()
         return body
 
     def get_history(self, alarm_id, q=None):
